package main

import (
	"context"
	"errors"
	"net/http"
	"os"
	"os/signal"
	"strconv"
	"syscall"
	"time"

	"git.anthrove.art/Anthrove/otter-cage/config"
	"git.anthrove.art/Anthrove/otter-cage/docs"
	v1 "git.anthrove.art/Anthrove/otter-cage/internal/api/v1"
	"git.anthrove.art/Anthrove/otter-cage/internal/utils"
	"git.anthrove.art/Anthrove/otter-cage/internal/vault"
	"git.anthrove.art/Anthrove/otter-space-sdk/v5/pkg/database"
	"git.anthrove.art/Anthrove/otter-space-sdk/v5/pkg/models"
	"github.com/gin-gonic/gin"
	log "github.com/sirupsen/logrus"
	swaggerFiles "github.com/swaggo/files"
	ginSwagger "github.com/swaggo/gin-swagger"
	"go.opentelemetry.io/contrib/instrumentation/github.com/gin-gonic/gin/otelgin"
	"go.opentelemetry.io/otel"
)

var otterCage config.OtterCage
var databaseConfig models.DatabaseConfig

var tracer = otel.Tracer("git.anthrove.art/Anthrove/otter-cage/cmd/otter-cage")

func init() {
	var err error
	ctx := context.Background()
	ctx, span := tracer.Start(ctx, "init")
	defer span.End()

	otterCage, err = config.LoadConfig[config.OtterCage](otterCage)
	if err != nil {
		log.WithError(err).Panic("cant load otterCage")
	}

	databaseConfig, err = config.LoadConfig[models.DatabaseConfig](databaseConfig)
	if err != nil {
		log.WithError(err).Panic("cant load databaseConfig")
	}
}

//	@title			Anthrove Otter Cage API
//	@version		1.0
//	@description	This service handles the management of secrets

//	@contact.name	API Support
//	@contact.url	http://anthrove.art/support
//	@contact.email	support@anthrove.art

// @BasePath	/api/v1
func main() {
	// ctx is the root context for the main function
	var ctx = context.Background()

	// Set up the logger based on the configuration
	utils.SetupLogger(otterCage.LogLevel, otterCage.LogFormat)

	ctx, span := tracer.Start(ctx, "main")
	defer span.End()

	log.WithContext(ctx).WithField("version", os.Getenv("VERSION")).Info("Starting OtterCage")

	// Connect do OtterSpace
	err := database.Connect(ctx, databaseConfig)
	if err != nil {
		log.Panic(err)
	}

	// TEST
	err = vault.CreateDatabase(ctx, "./database.kdbx", "aaaaaaaaa")
	if err != nil {
		log.WithError(err).Panic()
	}

	timeNow := time.Now()
	for i := range 500 {
		log.WithContext(ctx).WithField("i", i).Info("Adding API key")
		apiKey := vault.AddAPIKey(nil, "aaaaaaaaa", models.UserID(strconv.Itoa(i)), "aaaaaaaaaaaaaaaaaaaaaaaaa", "im-a-key")
		err := vault.AppendData(ctx, "./database.kdbx", "aaaaaaaaa", apiKey)
		if err != nil {
			log.WithError(err).Panic("ups")
		}

	}
	log.WithContext(ctx).WithField("duration", time.Since(timeNow)).Info("Finished")

	// TEST

	ginRouter := setupRouter(ctx)

	server := &http.Server{
		Addr:    ":8080",
		Handler: ginRouter,
	}

	go startServer(ctx, server)

	log.WithContext(ctx).Info("otter cage is running and ready to accept connections")

	waitForShutdown(ctx, server)
}

func setupRouter(ctx context.Context) *gin.Engine {
	ginRouter := gin.Default()
	ginRouter.Use(otelgin.Middleware("manager"))

	apiRouter := v1.NewAPIRouter(ginRouter.Group("/api/v1"))
	apiRouter.InitializeRoutes()

	if otterCage.Debug {
		gin.SetMode(gin.DebugMode)
		docs.SwaggerInfo.BasePath = "/api/v1"
		ginRouter.GET("/swagger/*any", ginSwagger.WrapHandler(swaggerFiles.Handler))
	}

	return ginRouter
}

func startServer(ctx context.Context, server *http.Server) {
	if err := server.ListenAndServe(); err != nil && !errors.Is(err, http.ErrServerClosed) {
		log.WithContext(ctx).WithError(err).Fatal("Failed to run server")
	}
}

func waitForShutdown(ctx context.Context, server *http.Server) {
	quit := make(chan os.Signal, 1)
	signal.Notify(quit, syscall.SIGINT, syscall.SIGTERM)
	<-quit
	log.WithContext(ctx).Info("Shutting down manager...")

	shutdownCtx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
	defer cancel()

	if err := server.Shutdown(shutdownCtx); err != nil {
		log.WithContext(ctx).WithError(err).Fatal("manager forced to shutdown")
	}
}
